package org.maltparser.core.dataformat.reader;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URL;

import org.maltparser.core.dataformat.DataFormatException;
import org.maltparser.core.exception.MaltChainedException;
import org.maltparser.core.sentence.Sentence;

/**
 *  
 *
 * @author Johan Hall
 * @since 1.0
**/
public class TabReader implements DataReader {
	private Sentence sentence;
	private BufferedReader reader;
	private int sentenceCount;
	
	public TabReader(Sentence sentence) {
		setSentence(sentence);
	}
	
	public void initialize(String fileName, String charsetName) throws MaltChainedException {
		try {
			initialize(new InputStreamReader(new FileInputStream(fileName),charsetName));
		}catch (FileNotFoundException e) {
			throw new DataFormatException("The input file '"+fileName+"' cannot be found. ", e);
		} catch (UnsupportedEncodingException e) {
			throw new DataFormatException("The character encoding set '"+charsetName+"' isn't supported. ", e);
		}
	}
	public void initialize(URL url) throws MaltChainedException {
		try {
			initialize(new InputStreamReader(url.openStream()));
		} catch (IOException e) {
			throw new DataFormatException("The URL '"+url.toString()+"' cannot be opened. ", e);
		}
	}
	public void initialize(InputStreamReader isr) throws MaltChainedException {
		setReader(new BufferedReader(isr));
		setSentenceCount(0);
	}
	
	public void readEpilog() {}

	public boolean readSentence() throws MaltChainedException  {
		String line = null;
		sentence.clearSentence();
		try {
			while (true) {
				line = reader.readLine();
				if (line == null) {
					if (sentence.hasTokens()) {
						sentenceCount++;
					}
					return false;
				} else if (line.length() == 0) {
					if (sentence.hasTokens()) {
						sentenceCount++;
					}
					return true;
				} else {
					sentence.setToken(line);
				}
			}
		}  catch (IOException e) {
			throw new DataFormatException("Error when reading from the input file.", e);
		} 
	}
	
	public void readProlog() {}

	
	public BufferedReader getReader() {
		return reader;
	}


	public void setReader(BufferedReader reader) {
		this.reader = reader;
	}


	public Sentence getSentence() {
		return sentence;
	}

	public void setSentence(Sentence sentence) {
		this.sentence = sentence;
	}

	public int getSentenceCount() {
		return sentenceCount;
	}

	public void setSentenceCount(int sentenceCount) {
		this.sentenceCount = sentenceCount;
	}
	
	public void close() throws MaltChainedException {
		try {
			reader.close();
		}   catch (IOException e) {
			throw new DataFormatException("Error when closing the input file.", e);
		} 
	}
}
